/*****************************************************************************
 *
 *   Copyright(C) 2011, Embedded Artists AB
 *   All rights reserved.
 *
 ******************************************************************************
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * Embedded Artists AB assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. Embedded Artists AB
 * reserves the right to make changes in the software without
 * notification. Embedded Artists AB also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
 *****************************************************************************/
package com.embeddedartists.aoa.nodes;



import android.app.Activity;
import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.util.Log;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.LinearLayout;
import android.widget.TextView;
import android.widget.ToggleButton;

public class NodeActivity extends Activity implements Node.OnValueChangeListener, 
NodeList.OnListChangeListener, OnClickListener {

	/**************************************************************************
	 * Private variables 
	 *************************************************************************/	
	
	private int nodeId = 0;
	private LinearLayout tempLayout;
	private LinearLayout btnLayout;
	private LinearLayout rgbLayout;
	private LinearLayout ledLayout;
	private LinearLayout lightLayout;
	
	private TextView tempValue;
	private TextView lightValue;
	private TextView btnValue;
	
	private NodeList nodeList;
	
	private static final int VALUE_UPDATED = 0;
	
	/**************************************************************************
	 * Overridden methods 
	 *************************************************************************/	
	
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		
        setContentView(R.layout.node);
        
        Log.i("EA AOA CAN", "NodeActivity.onCreate");
        
        tempLayout = (LinearLayout)findViewById(R.id.tempLayout);
        btnLayout = (LinearLayout)findViewById(R.id.btnLayout);
        rgbLayout = (LinearLayout)findViewById(R.id.rgbLayout);
        ledLayout = (LinearLayout)findViewById(R.id.ledLayout);
        lightLayout = (LinearLayout)findViewById(R.id.lightLayout);
        
        tempValue = (TextView)findViewById(R.id.tempValue);
        lightValue = (TextView)findViewById(R.id.lightValue);
        btnValue = (TextView)findViewById(R.id.btn1Value);
        
        ((Button)findViewById(R.id.redBtn)).setOnClickListener(this);
        ((Button)findViewById(R.id.greenBtn)).setOnClickListener(this);
        ((Button)findViewById(R.id.blueBtn)).setOnClickListener(this);        
        ((Button)findViewById(R.id.ledBtn)).setOnClickListener(this);
        
        nodeList = NodeList.getInstance();
                
	}

	@Override
	protected void onPause() {
		super.onPause();
		
		Log.i("EA AOA CAN", "NodeActivity.onPause");
		
		nodeList.removeOnListChangeListener(this);
		
		Node n = nodeList.getNodeWithId(nodeId);
		if (n != null) {
			n.removeOnValueChangeListener(this);
		}
			
	}

	@Override
	protected void onResume() {
		super.onResume();
		
		
		Log.i("EA AOA CAN", "NodeActivity.onResume");
		
		// get the Node ID associated with this NodeActivity. 
        Intent intent = getIntent();
        if (intent != null) {
        	nodeId = intent.getIntExtra("nodeId", -1);
        
        	setTitle("Node: "+ nodeId);
        	        	
        	Node n = nodeList.getNodeWithId(nodeId);
        	int[] caps = n.getCapabilities();
        	enableLayouts(caps);
        	
        }		
		
    	nodeList.addOnListChangeListener(this);
		
    	Node n = nodeList.getNodeWithId(nodeId);
    	n.addOnValueChangeListener(this);
    	
    	int[] caps = n.getCapabilities();

    	for (int i = 0; i < caps.length; i++) {
    		updateValue(n, caps[i]);
    	}
    	
	}


	/**
	 * A value has changed on the Node associated with this NodeActivity
	 */
	public void onValueChange(int capId) {
		Message m = Message.obtain(handler, VALUE_UPDATED);
		m.arg1 = capId;
		handler.sendMessage(m);						
	}
	
	/**
	 * The node list has changed.
	 */
	public void onListChange() {
		Node n = nodeList.getNodeWithId(nodeId);
		if (n == null) {
			// node associated with this activity has been removed
			finish();
		}
		
	}	
	
	/**
	 * A button has been clicked
	 */
	public void onClick(View v) {
		
		Node n = nodeList.getNodeWithId(nodeId);
		if (n == null) {
			return;
		}		
		
		switch (v.getId()) {
		case R.id.redBtn:
			n.setRedLed( ((ToggleButton)v).isChecked());
				
			break;
		case R.id.greenBtn:
			n.setGreenLed( ((ToggleButton)v).isChecked());
					
			break;
		case R.id.blueBtn:
			n.setBlueLed( ((ToggleButton)v).isChecked());
						
			break;
			
		case R.id.ledBtn:
			n.setLed( ((ToggleButton)v).isChecked());
			break;
						
		}		
	}		
	
	/**************************************************************************
	 * Private methods  
	 *************************************************************************/	
	
	/**
	 * Enable layout/graphics for a specific capability/device
	 * 
	 * @param caps capabilities for the node
	 */	
	private void enableLayouts(int[] caps) {
		for (int i = 0; i < caps.length; i++) {
			switch (caps[i]) {
			case Node.DEV_TEMP:
				tempLayout.setVisibility(View.VISIBLE);
				break;
			case Node.DEV_LIGHT:
				lightLayout.setVisibility(View.VISIBLE);
				break;
			case Node.DEV_BTN:
				btnLayout.setVisibility(View.VISIBLE);
				break;
			case Node.DEV_RGB:
				rgbLayout.setVisibility(View.VISIBLE);
				break;
			case Node.DEV_LED:
				ledLayout.setVisibility(View.VISIBLE);
				break;
			}
		}
	}
	
	/**
	 * Update a value
	 * 
	 * @param n The node to get the value from
	 * @param capId capability ID
	 */	
	private void updateValue(Node n, int capId) {
		switch (capId) {
		case Node.DEV_TEMP:
			tempValue.setText(n.getValue(capId));
			break;
		case Node.DEV_LIGHT:
			lightValue.setText(n.getValue(capId));
			break;
		case Node.DEV_BTN:
			btnValue.setText(n.getValue(capId));
			break;
		}		
	}
	
	/**************************************************************************
	 * Anonymous classes 
	 *************************************************************************/	

	/*
	 * The Android UI Toolkit is not thread-safe which means that only the 
	 * UI Thread may manipulate the User Interface. 
	 * 
	 * A Handler allows a thread to send a message to another thread's 
	 * message queue. When the Handler is created it will be associated
	 * with the thread that is creating the Handler. For the Handler below
	 * this means the UI thread.
	 * 
	 * Since messages from the Accessory are received by a separate thread
	 * those messages must be sent to the Handler below which is responsible
	 * for updating the UI.
	 */
	
	
	private final Handler handler = new Handler() {

		@Override
		public void handleMessage(Message msg) {
			
			switch(msg.what) {
			case VALUE_UPDATED: 
								
				Node n = NodeList.getInstance().getNodeWithId(nodeId);
				updateValue(n, msg.arg1);
				
				break;
								
		
			}
		}
		
	};





}
